#include "Hand.h"


Hand::Hand()
{
	setMaxPlayers(MAX_PLAYERS);
	_MAX_BLIND = MAX_BLIND;
	_MAX_HANDS = MAX_HANDS;

	setPicker(NOT_SET);
	setPartner(NOT_SET);
	setHandNumber(NOT_SET);
	setCalledSuit("Not Set");
	setLeadSuit("NN");
	setDealer(0);
	setNumPlayers(0);

	setPlayTurn();
	setStarted(false);
	setState(LOGIN);
	setHandWinnerId(NOT_SET);

}

void Hand::reset(int deal, int numPlay) {
	// Initalize players

	setMaxPlayers(MAX_PLAYERS);
	_MAX_BLIND = MAX_BLIND;
	_MAX_HANDS = MAX_HANDS;

	setPicker(NOT_SET);
	setPartner(NOT_SET);
	setHandNumber(NOT_SET);
	setCalledSuit("Not Set");
	setDealer(deal);
	setPlayTurn();
	setNumPlayers(numPlay);
	setNumBlindCards(2);
	setStarted(false);
	setHandWinnerId(NOT_SET);
	setHandWinner(NOT_SET);
	setGameWinner("?");
	setLeadSuit("NN");

	for (int i = 1; i <= MAX_HANDS; i++) {
		SUITLEAD[i] = 'N';
		HANDWINNER[i] = NOT_SET;
	}


	for (int z = 0; z < getNumPlayers(); z++) {
		Player * p1 = updatePlayerById(z);
		p1->resetNumCards();
		p1->setPassed(0);
		p1->setPicked(0);
		p1->setPoints(0);
		p1->setTeamPoints(0);
	}
}

// Players are 

void Hand::play(int maxPlayers, int maxBlind, int maxHand)
{
	setNumBlindCards(maxBlind);
	setNumPlayerCards(maxHand);
	setMaxPlayers(maxPlayers);

	int i;
	int passCount = 0;
	String cardId = "";
	int cardsPlayed = 0;

	//Serial.println("*******************************************************");
	//Serial.print("Get Game State = ");
	//Serial.println(getState());
	//Serial.println("*******************************************************");

	switch (getState()) {
	case LOGIN:
		if (getNumPlayers() != getMaxPlayers())
		{
			//Serial.println("*******************************************************");
			//Serial.print("Game State NOT CHANGED = ");
			//Serial.println(getState());
			//Serial.println("*******************************************************");
			return;
		}
		else {
			moveNextState();
		}
		break;

	case START:
		if (isStarted()) {
			//Serial.println("*******************************************************");
			//Serial.print("Game Started - State = START = ");
			//Serial.println(getState());
			//Serial.println("*******************************************************");
			moveNextState();
		}
		break;
	case DEAL:
		//Serial.println("*******************************************************");
		//Serial.print("Game Not Started - State = DEAL = ");
		//Serial.println(getState());
		//Serial.println("*******************************************************");

		// Shuffle the Deck
		shuffleDeck();
		// Deal the cards
		dealCards();
		printHand();
		//setStarted(true);
		Serial.println("*******************************************************");
		Serial.println(" Setting hand to 1");
		setHandNumber(1);
		Serial.println("*******************************************************");


		moveNextState();
		break;
	case PICKING:
		// Check for a picker if no picker end game
		for (i = 0; i < getNumPlayers(); i++) {
			if (getPlayerById(i).getPicked() == 1) {
				// set player turn to dealer + 1;
				setPlayTurn();
				moveNextState();

				// update player message
				setAllPlayerMsg(WAITING);
			}
			if (getPlayerById(i).getPassed() == 1) {
				passCount++;
			}
		}
		if (passCount == getNumPlayers()) {
			// Reset game and start no hand.  
			// TODO could do leaster			
			setState(GWINNER);
		}
		break;

	case HAND:
		// check that every player played a card.

		//Serial.println("********************************************");
		//Serial.print("In HAND");
		//Serial.println(getState()-PICKING);
		//Serial.println("********************************************");

		for (i = 0; i < getNumPlayers(); i++) {
			String card = getPlayerById(i).findCardNameByHand(getHandNumber());


			//Serial.println("********************************************");
			//Serial.print("Player ");
			//Serial.print(i);
			//Serial.print(" played card ");
			//Serial.println(card);
			//Serial.println("********************************************");

			if (card != "back") {
				cardsPlayed++;
				cardId = card;
			}
		}
		if (cardsPlayed == 1) {
			setLeadSuit(cardId);
			//Serial.println("********************************************");
			//Serial.print("Suit lead ");
			//Serial.println(getLeadSuit());
			//Serial.println("********************************************");
		}
		// count points
		if (cardsPlayed == getNumPlayers()) {
			// Determine hand winner
			int highPlayer = determineWinner(getHandNumber());
			setHandWinner(highPlayer);

			// Count points.
			countPoints(getHandNumber());
			setPlayTurnByHand(getHandNumber());
			moveNextState();

			//Serial.println("********************************************");
			//Serial.print("End of hand ");
			//Serial.println(getHandNumber());
			//Serial.println("********************************************");

		}
		break;
	case CONFIRM:
		// Wait for user input to move to the next state
		break;
	case HWINNER:
		if (getHandNumber() < getNumPlayerCards()) {

			setHandNumber(getHandNumber() + 1);
			setHandWinnerId(NOT_SET);
			setLeadSuit("NN");
			Serial.println("********************************************");
			Serial.print("moving to new hand ");
			Serial.println(getHandNumber());
			Serial.println("********************************************");
			setState(HAND);
		}
		else {
			setGameWinner();
			moveNextState();
		}
		break;
	case GWINNER:
		Serial.println("Someone Won!!!! Count the points.");

		break;
	case END:
		Serial.println("Someone Won!!!! Count the points.");
		reset(getDealer() + 1, getNumPlayers());

		setStarted(false);
		setState(LOGIN);

		break;
	default: break;
	}

}

