#pragma once

#include "Deck.h"
#include "Player.h"
#include "Card.h"
#include <WString.h>

#define NOT_SET -1
#define MAX_PLAYERS 5
#define MAX_BLIND 4
#define MAX_HANDS 10

enum STATE { LOGIN, START, DEAL, PICKING, HAND, CONFIRM, HWINNER, GWINNER, END };
int HANDWINNER[MAX_HANDS + 1] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
char SUITLEAD[MAX_HANDS + 1] = { 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N' };
enum PLAYERMESSAGE { NOTSET, WAITING, PICKED, PASSED, PLAYED, PARTNER, WINHAND };


class IGame
{
private:
	String _calledsuit;
	int _dealer;
	bool _started;
	int _state;
	String _gamewinner;
	int _handnumber;
	String _handwinner;
	int _handwinnerid;
	char _leadsuit;
	int _numblindcards;
	int _numplayercards;
	int _numplayers;
	int _partner;
	int _picker;
	int _playerturn;

	Deck _deck;
	Card * _blindcards[MAX_BLIND];
	Player _players[MAX_PLAYERS];
	Card card;

public:
	IGame();
	virtual void play(int, int, int) = 0;

	int _MAX_PLAYERS = MAX_PLAYERS;
	int _MAX_BLIND = MAX_BLIND;
	int _MAX_HANDS = MAX_HANDS;

	void setMaxPlayers(int);
	int getMaxPlayers();

	Player * getNextPlayer();
	Player getPlayerById(int);
	Player * updatePlayerById(int);
	int advanceNextTurn();
	void addPlayer(int, String);
	void initPlayers();
	void deletePlayer(int, String);

	void printHand();
	String getHandJSON(int);
	void shuffleDeck();
	void dealCards();
	void addBlind(int);
	String getBlindJSON();
	String getStatJSON(int);
	String getPlayerJSON();

	String getCalledSuit();
	void setCalledSuit(String);

	int getDealer();
	void setDealer(int);

	bool isStarted();
	void setStarted(bool);

	int getState();
	void setState(int);
	void moveNextState();

	void setHandNumber(int);
	int getHandNumber();

	int getNumPlayers();
	void setNumPlayers(int);

	void setPartner(int);
	int getPartner();
	String getPartnerName();

	void setPicker(int);
	int getPicker();
	String getPickerName();

	int getPlayerTurn();
	void setPlayTurn();
	void setPlayTurnByHand(int);

	int getNumBlindCards();
	void setNumBlindCards(int);

	int getNumPlayerCards();
	void setNumPlayerCards(int);

	void setLeadSuit(String);

	char getLeadSuit();
	char getSuitById(String);

	int trumpRank(String);

	void setAllPlayerMsg(int);

	int determineWinner(int);
	void countPoints(int);

	void setHandWinner(int);
	String getHandWinner();

	int getHandWinnerId();
	void setHandWinnerId(int);

	void setGameWinner();
	void setGameWinner(String);
	String getGameWinner();
};

