#include "IGame.h"

IGame::IGame()
{

}

void IGame::moveNextState() {
	if (_state == END) {
		setState(LOGIN);
	}
	else {
		_state++;
	}
	Serial.println("*******************************************************");
	Serial.print("Game State = ");
	Serial.println(_state);
	Serial.println("*******************************************************");
}
Player * IGame::getNextPlayer()
{
	_playerturn++;
	if (_playerturn >= _MAX_PLAYERS)
	{
		_playerturn = 0;
	}
	return &_players[_playerturn];

}

int IGame::advanceNextTurn()
{
	_playerturn++;
	if (_playerturn >= _MAX_PLAYERS)
	{
		_playerturn = 0;
	}
	return _playerturn;

}

void IGame::addPlayer(int index, String name)
{
	if (index >= 0 && index < _MAX_PLAYERS) {
		_players[index].setActive(true);
		_players[index].setName(name);
		_players[index].setPicked(0);
		_players[index].setPassed(0);

		_numplayers++;
	}
	else {
		Serial.print("Error in addPlayer index = ");
		Serial.println(index);
	}
}

void IGame::initPlayers()
{
	for (int i = 0; i < _MAX_PLAYERS; i++)
	{
		_players[i] = Player("");
	}

}

void IGame::shuffleDeck() {
	_deck.shuffle();
}

String IGame::getHandJSON(int playerid) {
	if (playerid >= 0 && playerid < _MAX_PLAYERS) {
		return _players[playerid].getHandJSON();
	}
	return "";
}

String IGame::getBlindJSON() {

	String blind = "\"blind\" : [";

	bool first = true;
	for (int i = 0; i < _numblindcards; i++) {
		Card * card = _blindcards[i];
		if (first) {
			first = false;
		}
		else {
			blind += ",";
		}
		blind += "{\"card\":\"" + card->Card2Str() + "\"";
		blind += "}";
	}

	blind += "]";

	return blind;
}

String IGame::getStatJSON(int playerid) {

	String message = "";

	if (playerid < 0) {
		return message;
	}

	message += "\"turn\":";
	message += getPlayerTurn();
	message += ",";

	message += "\"passed\":";
	message += getPlayerById(playerid).getPassed();
	message += ",";

	message += "\"picked\":";
	message += getPlayerById(playerid).getPicked();
	message += ",";

	message += "\"state\":";
	message += getState();
	message += ",";

	message += "\"handnum\":";
	message += getHandNumber();
	message += ",";

	message += "\"numplaycrd\":";
	message += getNumPlayerCards();
	message += ",";

	message += "\"numblindcrd\":";
	message += getNumBlindCards();
	message += ",";

	message += "\"points\":";
	message += getPlayerById(playerid).getPoints();
	message += ",";

	// TODO
	message += "\"tpoints\":";
	message += getPlayerById(playerid).getTeamPoints();
	message += ",";

	message += "\"hwin\":\"";
	message += getHandWinner();
	message += "\",";

	if (getHandNumber() > 0) {
		message += "\"hwinid\":";
		message += getHandWinnerId();
		message += ",";
	}
	message += "\"gwin\":\"";
	message += getGameWinner();
	message += "\",";

	message += "\"csuit\":\"";
	message += getCalledSuit();
	message += "\",";

	message += "\"lsuit\":\"";
	message += getLeadSuit();
	message += "\",";

	message += "\"picker\": \"";
	message += getPickerName();
	message += "\",";
	message += "\"partner\": \"";
	message += getPartnerName();
	message += "\"";

	return message;
}

String IGame::getPlayerJSON() {
	Serial.println("***************************************************");
	Serial.print("In getPlayerJSON() Hand # = ");
	bool firstPlayer = true;
	int hand = getHandNumber();
	Serial.println(hand);


	String message = "";
	message += "\"players\":[";
	for (int i = 0; i < _MAX_PLAYERS; i++)
	{
		Player p1 = getPlayerById(i);
		Serial.print("Player id = ");
		Serial.println(i);
		Serial.print("Player active = ");
		Serial.println(p1.getActive());

		if (p1.getActive() == true) {
			String cardName = p1.findCardNameByHand(hand);
			if (firstPlayer) {
				firstPlayer = false;
			}
			else {
				message += ",";
			}
			message += "{\"name\":\"" + p1.getName() + "\",";
			message += "\"card\":\"" + cardName + "\",";
			message += "\"message\":\"" + p1.getPlayerMsg() + "\"}";
		}
		else {
			if (firstPlayer) {
				firstPlayer = false;
			}
			else {
				message += ",";
			}
			message += "{\"name\":\"<br>\"}";
		}
	}
	message += "]";

	return message;
	Serial.println(message);
}

void IGame::printHand()
{
	for (int i = 0; i < _numplayers; i++)
	{
		_players[i].printHand();
	}

}

void IGame::dealCards()
{
	// Deal the cards
	int i = 0;
	int blindCardCount = 0;
	int cardsPerPlayer = (MAX_CARDS - _numblindcards) / _numplayers;
	while (i < MAX_CARDS)
	{
		for (int x = 0; x < _numplayers; x++)
		{
			Player * player = getNextPlayer();

			if ((cardsPerPlayer - player->getNumCards()) == 3)
			{
				player->setCard(_deck.getOneCard());
				player->setCard(_deck.getOneCard());
				player->setCard(_deck.getOneCard());
				i = i + 3;
			}
			else
			{
				player->setCard(_deck.getOneCard());
				player->setCard(_deck.getOneCard());

				i = i + 2;
			}
		}
		// Always add 2 cards to the blind
		if (blindCardCount < _numblindcards)
		{
			_blindcards[blindCardCount++] = _deck.getOneCard();
			_blindcards[blindCardCount++] = _deck.getOneCard();
			i = i + 2;
		}
	}
}

void IGame::addBlind(int picker)
{
	if ((picker >= 0) && (picker < _MAX_PLAYERS)) {
		Player &p = _players[picker];

		for (int i = 0; i < _numblindcards; i++)
		{
			p.setCard(_blindcards[i]);
		}
	}
}



void IGame::setPlayTurn()
{
	_playerturn = getDealer() + 1;
	if (_playerturn >= _MAX_PLAYERS)
	{
		_playerturn = 0;
	}
}

// I need to set player turns after a winning hand
void IGame::setPlayTurnByHand(int hand)
{
	_playerturn = getHandWinnerId();
}


Player IGame::getPlayerById(int index)
{
	if (index >= 0 && index < _MAX_PLAYERS) {
		return _players[index];
	}
	Serial.print("Error in getPlayerByID index = ");
	Serial.println(index);
	return _players[0];
}

Player * IGame::updatePlayerById(int index)
{
	if (index >= 0 && index < _MAX_PLAYERS) {
		return &_players[index];
	}
	Serial.print("Error in updatePlayerByID index = ");
	Serial.println(index);
	return &_players[0];
}

void IGame::deletePlayer(int index, String playerName)
{
	Player p1 = getPlayerById(index);
	if (p1.getName() == playerName) {
		_players[index].setActive(false);
		_players[index].setName("");
		_players[index].setPicked(0);
		_players[index].setPassed(0);

		_numplayers--;
	}
}


String IGame::getCalledSuit() {
	return _calledsuit;
}
void IGame::setCalledSuit(String cs) {
	_calledsuit = cs;
}

int IGame::getDealer() {
	return _dealer;
}
void IGame::setDealer(int deal) {
	_dealer = deal;
	if (_dealer >= _MAX_PLAYERS)
	{
		_dealer = 0;
	}
}

bool IGame::isStarted() {
	return _started;
}
void IGame::setStarted(bool gs) {
	_started = gs;
}

int IGame::getState() {
	return _state;
}
void IGame::setState(int state) {
	_state = state;
}

void IGame::setHandNumber(int hand) {
	if (hand > 0 && hand <= _MAX_HANDS)
		_handnumber = hand;
	Serial.println("*******************************************************");
	Serial.print("IN setHandNumber() hand set = ");
	Serial.println(_handnumber);
	Serial.print("IN setHandNumber() _MAX_HAND set = ");
	Serial.println(_MAX_HANDS);
	Serial.println("*******************************************************");

}

int IGame::getHandNumber() {
	return _handnumber;
}


int IGame::getNumPlayers()
{
	return _numplayers;
}
void IGame::setNumPlayers(int num) {
	_numplayers = num;

	Serial.print("setNumPlayers = ");
	Serial.println(num);
}

void IGame::setMaxPlayers(int num) {
	_MAX_PLAYERS = num;
}
int IGame::getMaxPlayers()
{
	return _MAX_PLAYERS;
}

void IGame::setPartner(int part) {
	_partner = part;
}
int IGame::getPartner() {
	return _partner;
}
String IGame::getPartnerName() {
	if (_partner == NOT_SET) {
		return "Not Set";
	}
	else {
		return getPlayerById(_partner).getName();
	}
}

void IGame::setPicker(int pick) {
	_picker = pick;
}
int IGame::getPicker() {
	return _picker;
}
String IGame::getPickerName() {

	if (_picker == NOT_SET) {
		return "Not Set";
	}
	else {
		return getPlayerById(_picker).getName();
	}

}
int IGame::getPlayerTurn() {
	return _playerturn;

}

char IGame::getLeadSuit() {
	Serial.println("In getLeadSuit() **************************************");
	char lead = SUITLEAD[getHandNumber()];
	if (!lead) {
		lead = 'N';
	}
	Serial.print("Lead suit = (");
	Serial.print(lead);
	Serial.print(")");
	return lead;
}
void IGame::setLeadSuit(String cardId) {

	Serial.print("In setLeadSuit with ");
	Serial.println(cardId);

	_leadsuit = getSuitById(cardId);
	Serial.print("Hand = ");
	Serial.print(getHandNumber());
	Serial.print(" leadsuit = ");
	Serial.println(_leadsuit);
	SUITLEAD[getHandNumber()] = _leadsuit;
}

char IGame::getSuitById(String cardId) {

	bool trump = false;

	for (int i = 0; i < TRUMP_MAX; i++) {
		if (TRUMP[i].compareTo(cardId) == 0) {
			return 'T';
		}
	}
	return cardId.charAt(1);
}
void IGame::setAllPlayerMsg(int msgId) {

	for (int i = 0; i < getNumPlayers(); i++) {
		updatePlayerById(i)->setPlayerMsg(msgId);
	}
}

int IGame::determineWinner(int hand) {
	char leadSuit = getLeadSuit();
	// if trump is player find high trump

	int highRank = -1;
	int highPlayer = -1;
	for (int i = 0; i < getNumPlayers(); i++) {
		String cardId = getPlayerById(i).findCardNameByHand(hand);

		Serial.println("In determineWinner trump");

		int rank = trumpRank(cardId);
		if (rank > highRank) {
			highPlayer = i;
			highRank = rank;
		}
	}
	// else find high of suit lead
	if (highRank == -1) {
		// trump was not found
		for (int i = 0; i < getNumPlayers(); i++) {
			String cardId = getPlayerById(i).findCardNameByHand(hand);

			Serial.println("In determineWinner lead suit ");
			int rank = card.cardRank(leadSuit, cardId);
			if (rank > highRank) {
				highPlayer = i;
				highRank = rank;
			}
		}
	}

	return highPlayer;
}

void IGame::countPoints(int hand) {
	if (hand < 1) {
		Serial.println("Error - Error - Error invalid hand number - Error Error Error");
		return;
	}
	int pid = getHandWinnerId();
	if (pid < 0) {
		Serial.println("Error - Error - Error invalid player id - Error Error Error");
		return;
	}
	Player * winner = updatePlayerById(pid);

	Serial.print("Player ");
	Serial.print(winner->getName());
	Serial.println(" Won!!!!!");
	for (int i = 0; i < getNumPlayers(); i++) {
		Player * p1 = updatePlayerById(i);
		String cardId = p1->findCardNameByHand(hand);
		winner->addPoints(cardId);

	}

	// Count team points
	if ((getNumPlayers() == 5) && (getPartner() != NOT_SET)) {
		Player * partner = updatePlayerById(getPartner());
		Player * picker = updatePlayerById(getPicker());
		// Set team points for picker and partner
		partner->setTeamPoints(partner->getPoints() + picker->getPoints());
		picker->setTeamPoints(partner->getPoints() + picker->getPoints());
	}
	else {
		Player * picker = updatePlayerById(getPicker());
		picker->setTeamPoints(picker->getPoints());
	}

	// Count non-picker team points only if partner is known
	// or no partner
	if (((getNumPlayers() == 5) && (getPartner() != NOT_SET)) ||
		((getNumPlayers() == 3) || (getNumPlayers() == 4))) {

		int teamPoints = 0;

		// Count up all points for non-picker/partners
		for (int i = 0; i < getNumPlayers(); i++) {
			if ((i == getPicker()) || (i == getPartner())) {
				continue;
			}

			Player * p1 = updatePlayerById(i);
			teamPoints += p1->getPoints();
		}

		for (int i = 0; i < getNumPlayers(); i++) {
			if ((i == getPicker()) || (i == getPartner())) {
				continue;
			}

			Player * p1 = updatePlayerById(i);
			p1->setTeamPoints(teamPoints);
		}
	}
}

int IGame::getNumBlindCards() {
	return _numblindcards;
}
void IGame::setNumBlindCards(int num) {
	_numblindcards = num;
}

int IGame::getNumPlayerCards() {
	return _numplayercards;
}
void IGame::setNumPlayerCards(int num) {
	_numplayercards = num;
}

void IGame::setHandWinner(int playerId) {
	if (playerId >= 0) {
		_handwinnerid = playerId;
		HANDWINNER[getHandNumber()] = playerId;

		_handwinner = getPlayerById(playerId).getName();
	}
	else {
		Serial.print("Error in sethandWinner, invalid playerId = ");
		Serial.println(playerId);
	}
}
String IGame::getHandWinner() {
	return _handwinner;
}

int IGame::getHandWinnerId() {
	return _handwinnerid;
}

void IGame::setHandWinnerId(int id) {
	if (id >= 0 && id < _MAX_PLAYERS) {
		_handwinnerid = id;
	}
	else {
		Serial.print("Error in setHandWinnerId, id = ");
		Serial.println(id);
		_handwinnerid = NOT_SET;
	}
}

int IGame::trumpRank(String cardId) {
	for (int i = 0; i < TRUMP_MAX; i++) {
		if (TRUMP[i].compareTo(cardId) == 0) {
			return i;
		}
	}
	return -1;
}

void IGame::setGameWinner(String winner) {
	_gamewinner = winner;
}

void IGame::setGameWinner() {
	_gamewinner = "";
	int highPoints = 0;
	for (int i = 0; i < getNumPlayers(); i++) {
		Player p1 = getPlayerById(i);

		if (p1.getTeamPoints() > highPoints) {
			_gamewinner = p1.getName();
			highPoints = p1.getTeamPoints();
		}
		else if (p1.getTeamPoints() == highPoints) {
			if (_gamewinner.length() > 0) {
				_gamewinner = _gamewinner + "/" + p1.getName();
			}
			else {
				_gamewinner = p1.getName();
			}
		}
	}
}
String IGame::getGameWinner() {
	return _gamewinner;
}