#include "Player.h"

Player::Player()
{
	//Serial.println("In Player Constructor");
	_name = "";
	_numberofcards = 0;
	_points = 0;
	_teampoints = 0;
	_active = false;
	_passed = false;
	_picked = false;
	setPlayerMsg(0);
	for (int i = 0; i < MAX_PLAYER_CARDS; i++) {
		_handplayed[i] = 0;
		_cardplayed[i] = -1;
	}
	//Serial.println("End Player Constructor");
}
Player::Player(String playername)
{
	//Serial.println("In Player Name Constructor");
	_name = playername;
	if (_name.length() > 6) {
		_name.substring(0, 5);
	}
	_numberofcards = 0;
	_points = 0;
	_active = 0;
	_passed = 0;
	_picked = 0;
	setPlayerMsg(1);
	for (int i = 0; i < MAX_PLAYER_CARDS; i++) {
		_handplayed[i] = 0;
		_cardplayed[i] = -1;
	}
	//Serial.println("End Player Name Constructor");
}


void Player::setCard(Card * card)
{
	//Serial.println("In Player setCard");
    _cards[_numberofcards] = card;
	_handplayed[_numberofcards] = 0;
	_cardplayed[_numberofcards] = -1;
    _numberofcards++;
	Serial.print("End Player = ");
	Serial.print(_name);
	Serial.print(" setCard = ");
	Serial.println(card->Card2Str());
}



void Player::updateCardById(int id, int handPlayed) {
	
	if(id >= 0)
		_handplayed[id] = handPlayed;
	if(handPlayed > 0)
		_cardplayed[handPlayed-1] = id;
}

void Player::updateCardPlayed(String cardValue, int handPlayed) {
	// Find the card in the list and update its handPlayed value
	bool found = false;
	Serial.print("In updateCardPlayed card = ");
	Serial.print(cardValue);
	Serial.print(" hand = ");
	Serial.println(handPlayed);

	for (int i = 0; i < MAX_PLAYER_CARDS;i++) {

		if (_cards[i]->Card2Str() == cardValue) {
			_handplayed[i] = handPlayed;
			if(handPlayed > 0)
				_cardplayed[handPlayed-1] = i;
			found = true;
			break;
		}
	}
	
}

String Player::findCardNameByHand(int hand) {
	
	if ((hand > 0) && (_cardplayed[hand - 1] >= 0)) {
		if(_cardplayed[hand - 1] >= 0)
			return _cards[_cardplayed[hand - 1]]->Card2Str();
	}

	return "back";
}

String Player::getHandJSON() {
	String hand = "\"mycards\" : [";
	bool firstCard = true;

	for (int i = 0; i < _numberofcards; i++) {
		Card * card = _cards[i];

		// If the card has not been played put it in the hand
		if (_handplayed[i] == 0) {
			if (!firstCard) {
				hand += ",";
			}
			firstCard = false;

			hand += "{\"card\":\"" + card->Card2Str() + "\",";
			hand += "\"status\":";
			hand += _handplayed[i];
			hand += "}";
			
		}
	}

	hand += "]";

	return hand;

}
void Player::printHand()
{
	Serial.println(_name);
  for (int i = 0; i < _numberofcards; i++)
  {
    Card * card = _cards[i];
	Serial.print(card->Card2Str());
	Serial.print(" ");
  }
  Serial.println();
}

String Player::getName()
{
	return _name;
}
void Player::setName(String nm)
{
	_name = nm;
}

int Player::getNumCards()
{
	return _numberofcards;
}

void Player::resetNumCards()
{
	_numberofcards = 0;
}

bool Player::getActive()
{
	return _active;
}
void Player::setActive(bool act)
{
	_active = act;
}

bool Player::getPassed()
{
	return _passed;
}
void Player::setPassed(bool pass)
{
	_passed = pass;
}

bool Player::getPicked()
{
	return _picked;
}
void Player::setPicked(bool pick)
{
	_picked = pick;
}

String Player::getPlayerMsg() {
	return _playermessage;
}
void Player::setPlayerMsg(int index) {
	switch (index) {
	case 1: _playermessage = "Waiting";
		break;
	case 2: _playermessage = "Picked";
		break;
	case 3: _playermessage = "Passed";
		break;
	case 4: _playermessage = "Played";
		break;
	case 5: _playermessage = "Partner";
		break;
	case 6: _playermessage = "Winner";
		break;
	default: _playermessage = "";
		break;
	}
}

void Player::setPoints(int points) {
	_points = points;
}

void Player::addPoints(int id) {
	_points += _cards[id]->getPoints();
}
void Player::addPoints(String cardId) {
	//Card card;
	_points += _cards[0]->getPoints(cardId);
}

int Player::getPoints() {
	return _points;
}

void Player::setTeamPoints(int points) {
	_teampoints = points;
}
void Player::addTeamPoints(int points) {
	_teampoints += points;
}
int Player::getTeamPoints() {
	return _teampoints;
}