/*
Name:		Sheepshead.ino
Created:	4/27/2018 10:26:45 PM
Author:	jeff
*/

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include "Player.h"
#include "Hand.h"
#include "FS.h"


const byte DNS_PORT = 53;
IPAddress apIP(192, 168, 1, 1);
IPAddress netMsk(255, 255, 255, 0);
DNSServer dnsServer;

/* hostname for mDNS. Should work at least on windows. Try http://192.168.1.1 */
const char *myHostname = "Sheepshead";

int currentPlayer = -1;
String playerMessage = "Waiting for more players to join";
IGame * game;
ESP8266WebServer server(80);

void setup(void) {
	delay(1000);
	Serial.begin(115200);
	Serial.println();
	Serial.print("Configuring access point...");

	//Serial.setDebugOutput(1);
	//WiFi.begin(ssid, password);
	//WiFi.begin();
	//Serial.println("");

	// TODO when setting up AP mode
	WiFi.mode(WIFI_AP);
	WiFi.softAPConfig(apIP, apIP, netMsk);
	WiFi.softAP(myHostname);

	delay(500); // Without delay I've seen the IP address blank 
	Serial.print("AP IP address: ");
	Serial.println(WiFi.softAPIP());


	// Wait for connection
	//while (WiFi.status() != WL_CONNECTED) {
	//	delay(500);
	//	Serial.print(".");
	//}
	//Serial.println("");
	//Serial.print("Connected to ");
	//Serial.println(ssid);
	//Serial.print("IP address: ");
	//Serial.println(WiFi.localIP());


	/*IPAddress myIP = WiFi.softAPIP();
	Serial.println("");
	Serial.print("Connected to ");
	Serial.println(myHostname);
	Serial.print("IP address: ");
	Serial.println(myIP);*/


	// if DNSServer is started with "*" for domain name, it will reply with
	// provided IP to all DNS request
	//dnsServer.setTTL(300);
	dnsServer.setErrorReplyCode(DNSReplyCode::NoError);
	dnsServer.start(DNS_PORT, "*", apIP);

	SPIFFS.begin();
	if (!SPIFFS.begin()) {
		Serial.println("SPIFFS failed to mount !\r\n");
	}

	//int GAME_MODE = 3;
	// Initialize game
	//if (GAME_MODE == 3) {
	game = new Hand();
	//}
	//else if (GAME_MODE == 4) {
	//	game = new FourHand();
	//}
	//else {
		//game = new FiveHand();
	//}
	//initSession();

	server.on("/", handleLogin);
	server.on("/join", handleJoin);
	server.on("/quit", handleQuit);
	server.on("/start", handleStart);
	server.on("/login.json", handleLoginJson);
	server.on("/gamestats.json", handleGameJson);
	server.on("/js/jquery-2.2.2.min.js", handleJS);
	server.on("/js/login.js", handleJS);


	server.on("/pick", handlePick);
	server.on("/pass", handlePass);
	server.on("/nextHand", handleNextHand);
	server.on("/gameEnd", handleGameEnd);
	server.on("/login", handleLogin);
	server.on("/sheepshead", handleGameStart);
	server.on("/player.json", handlePlayerJson);
	server.on("/playCard", handlePlayCard);
	server.on("/js/dragula.min.js", handleJS);
	server.on("/js/sheepshead.js", handleJS);

	//server.serveStatic("/img/cards/QC.png", SPIFFS, "/img/cards/QC.png", "max-age=86400");
	//server.serveStatic("/img/cards/QC.png", SPIFFS, "/img/cards/QS.png", "max-age=86400");
	//server.serveStatic("/img/cards/QC.png", SPIFFS, "/img/cards/QH.png", "max-age=86400");
	//server.serveStatic("/img/cards/QC.png", SPIFFS, "/img/cards/QD.png", "max-age=86400");

	server.on("/img/cards/QC.png", handleIMAGE);
	server.on("/img/cards/QS.png", handleIMAGE);
	server.on("/img/cards/QH.png", handleIMAGE);
	server.on("/img/cards/QD.png", handleIMAGE);
	server.on("/img/cards/JC.png", handleIMAGE);
	server.on("/img/cards/JS.png", handleIMAGE);
	server.on("/img/cards/JH.png", handleIMAGE);
	server.on("/img/cards/JD.png", handleIMAGE);
	server.on("/img/cards/AC.png", handleIMAGE);
	server.on("/img/cards/AS.png", handleIMAGE);
	server.on("/img/cards/AH.png", handleIMAGE);
	server.on("/img/cards/AD.png", handleIMAGE);
	server.on("/img/cards/TC.png", handleIMAGE);
	server.on("/img/cards/TS.png", handleIMAGE);
	server.on("/img/cards/TH.png", handleIMAGE);
	server.on("/img/cards/TD.png", handleIMAGE);
	server.on("/img/cards/KC.png", handleIMAGE);
	server.on("/img/cards/KS.png", handleIMAGE);
	server.on("/img/cards/KH.png", handleIMAGE);
	server.on("/img/cards/KD.png", handleIMAGE);
	server.on("/img/cards/9C.png", handleIMAGE);
	server.on("/img/cards/9S.png", handleIMAGE);
	server.on("/img/cards/9H.png", handleIMAGE);
	server.on("/img/cards/9D.png", handleIMAGE);
	server.on("/img/cards/8C.png", handleIMAGE);
	server.on("/img/cards/8S.png", handleIMAGE);
	server.on("/img/cards/8H.png", handleIMAGE);
	server.on("/img/cards/8D.png", handleIMAGE);
	server.on("/img/cards/7C.png", handleIMAGE);
	server.on("/img/cards/7S.png", handleIMAGE);
	server.on("/img/cards/7H.png", handleIMAGE);
	server.on("/img/cards/7D.png", handleIMAGE);
	server.on("/img/cards/back.png", handleIMAGE);

	server.onNotFound(handleNotFound);
	//here the list of headers to be recorded
	const char * headerkeys[] = { "User-Agent", "Cookie", "Cache-Control" };
	size_t headerkeyssize = sizeof(headerkeys) / sizeof(char*);
	//ask server to track these headers
	server.collectHeaders(headerkeys, headerkeyssize);

	// DNS Captive Portal page
	//handleLogin();
	server.begin();
	Serial.println("HTTP server started");

}

void loop(void) {
	//Serial.println("loop()");
	//DNS
	dnsServer.processNextRequest();
	server.handleClient();

	// determin how many players will play
	switch (game->getMaxPlayers()) {
	case 3:
		game->play(3, 2, 10);
		break;
	case 4:
		game->play(4, 4, 7);
		break;
	case 5:
		game->play(5, 2, 6);
		break;
	}

	//if (game->getState() == END) {
	//	handleLogin();
	//}
}


//Check if header is present and correct
bool is_authentified() {
	Serial.println("Enter is_authentified");
	String userName;

	if (server.hasHeader("Cookie")) {
		Serial.println("Found is_authentified cookie: ");
		String cookie = server.header("Cookie");
		Serial.print("Cookie = ");
		Serial.println(cookie);

		/*
		if (server.hasArg("SESSIONID")) {
			Serial.print("SESSIONID = ");
			Serial.println(server.arg("SESSIONID"));

			Serial.print("STATE = ");
			Serial.println(server.arg("STATE"));

			Serial.print("USERNAME = ");
			Serial.println(server.arg("USERNAME"));
		}
		*/

		int pos = cookie.indexOf("SESSIONID=", 0);
		Serial.print("Position session index = ");
		Serial.println(pos);
		String sessionStr = cookie.substring(pos + 10);
		Serial.print("SESSIONID = (");
		Serial.print(sessionStr);
		Serial.println(")");

		int sessionID = sessionStr.toInt();
		Serial.print("sessionID = ");
		Serial.println(sessionID);

		/*
		Serial.print("SessionId = ");
		Serial.println(sessionStr.substring(0, 1));
		Serial.print("State = ");
		pos = sessionStr.indexOf("STATE=", pos);
		Serial.println(sessionStr.substring(pos+6, pos+6+1));
		Serial.print("Username = ");
		pos = sessionStr.indexOf("USERNAME=", pos);
		Serial.println(sessionStr.substring(pos+9));
		*/


		pos = cookie.indexOf("USERNAME=", pos + 10);
		//pos = cookie.indexOf("USERNAME=", pos);
		Serial.print("Position USERNAME index = ");
		Serial.println(pos);
		if (pos >= 0) {
			userName = cookie.substring(pos + 9);
			//String arg = server.arg("SESSIONID");
			Serial.print("USERNAME = (");
			Serial.print(userName);
			Serial.println(")");


			if (findNamebySessionId(sessionID) == userName) {
				//if (cookie.indexOf("SESSIONID=") != -1) { //TODO
				//Serial.println("Authentification Successful");
				currentPlayer = sessionID;
				return true;
			}

		}
		else {
			Serial.println("USERNAME = (NOT FOUND) + " + cookie.substring(pos));
		}
	}
	Serial.println("Authentification Failed");
	return false;
}

void handleJoin() {

	if (server.hasArg("USERNAME")) {
		int players = game->getNumPlayers();
		String userName = server.arg("USERNAME");
		currentPlayer = findFreeSession(userName);

		//Serial.println(players);
		if (players >= game->getMaxPlayers() || currentPlayer == -1) {
			playerMessage = "We already have enough players for this game.";
			// TODO send player message
			server.send(200);
			return;
		}
		else if (currentPlayer == -2) {
			playerMessage = "User name (" + userName + ") has been used.";
			// TODO send player message
			server.send(200);
			return;
		}
		else {
			game->addPlayer(currentPlayer, server.arg("USERNAME"));
			Player p1 = game->getPlayerById(currentPlayer);

			int nPlayers = server.arg("NPLAYERS").toInt();
			Serial.print("Setting number of players = ");
			Serial.println(nPlayers);
			game->setMaxPlayers(nPlayers);

			String header = "HTTP/1.1 301 OK\r\nSet-Cookie: SESSIONID=";
			header += currentPlayer;
			header += "&STATE=";
			header += game->getState();
			header += "&USERNAME=";
			header += p1.getName();
			header += ";\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
			server.sendContent(header);
			//Serial.println("Log in Successful");
			//Serial.print("Welcome ");
			//Serial.println(p1.getName());
			playerMessage = "Welcome " + p1.getName();
			players++;

			Serial.println(playerMessage);
			server.send(200);
		}

	}
}
void handleQuit() {

	String playerName = server.arg("DISCONNECT");

	if (is_authentified()) {
		if (game->getState() < DEAL) {
			String session = server.arg("SESSIONID");
			int currentPlayer = session.toInt();
			String header = "HTTP/1.1 301 OK\r\nSet-Cookie: SESSIONID=-1";
			header += "&STATE=";
			header += game->getState();
			header += "&USERNAME=deleted;";
			header += "\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
			server.sendContent(header);

			game->deletePlayer(currentPlayer, playerName);

			playerMessage = "Goodbye " + playerName;
			return;
		}
		else {
			playerMessage = "Game has started";
		}
	}
	else {
		playerMessage = "Players " + playerName + " is not authorized";
	}
	server.send(200);
}

void handleStart() {

	if (is_authentified()) {
		Serial.print("Game Mode started !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		int players = game->getNumPlayers();
		if (players == game->getMaxPlayers()) {
			String userName = server.arg("USERNAME");
			String playerID = server.arg("SESSIONID");
			String header = "HTTP/1.1 301 OK\r\nSet-Cookie: SESSIONID=";
			header += playerID;
			header += "&USERNAME=";
			header += userName;
			header += "\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";

			//if (game->getState() == LOGIN) {
				//game->setState(DEAL);
			//}
			server.sendContent(header);

			playerMessage = "Sheepshead game starting, click Start button";

			//Serial.println(playerMessage);
			return;
		}
	}
}

void handlePick() {
	//Serial.print("in handlePick with URI = ");
	//Serial.println(server.uri());

	if (server.hasArg("playerPicked")) {
		//Serial.println("FOUND*******playerPicked*****************************************");
		//Serial.println("FOUND*******playerPicked***************************************");
		//Serial.println("FOUND*******playerPicked***************************************");
		//Serial.println("FOUND**********************************************************");

		//Serial.print("Player id picked = ");
		String picker = server.arg("playerPicked");
		//Serial.println(picker);
		game->setPicker(picker.toInt());
		Player * p1 = game->updatePlayerById(picker.toInt());
		p1->setPicked(1);
		p1->setPlayerMsg(2);

		game->setPicker(picker.toInt());
		// Set player Turn to dealer + 1 or the hand Winner
		//game->setPlayTurn(0);
		//game->setState(HAND1);
		String calledsuit = server.arg("csuit");
		//Serial.print("calledsuit set to = ");
		//Serial.println(calledsuit);
		game->setCalledSuit(calledsuit);

		// TODO get blind cards.
		game->addBlind(picker.toInt());

		for (int z = 0; z < server.args(); z++) {
			//Serial.println("Server Args ******************************************");
			//Serial.println(server.argName(z)); 
			//Serial.println(server.arg(z));
			//Serial.println("Server Args ******************************************");

		}
		// TODO mark cards that have been discards
		//for (int i = 0; i < game->getBlindSize(); i++) {
		//Serial.print("BLIND SIZE ============== ");
		//Serial.println(game->getNumBlindCards());
		for (int i = 2; i < server.args(); i++) {
			//Serial.println("CHECK FOR*****Blind Card******************************************");
			//Serial.println("CHECK FOR******Blind Card******************************************");

			String bname = server.argName(i);
			//Serial.print("Blind key = ");
			//Serial.println(bname);
			//if (server.hasArg(bname)) {

				//Serial.println("FOUND******Blind Card******************************************");
				//Serial.println("FOUND******Blind Card******************************************");
				//Serial.println("FOUND******Blind Card******************************************");
				//Serial.println("FOUND******Blind Card******************************************");

				//String cardId = server.arg("blind_" + i);
			String cardId = server.arg(i);
			//Serial.print("FOUND card = ");
			//Serial.print(cardId);

			//Player * p1 = game->updatePlayerById(picker.toInt());
			p1->updateCardById(cardId.toInt(), -1);
			p1->addPoints(cardId.toInt());
			//}
		}
		//Player * p1 = game->updatePlayerById(picker.toInt());
		p1->addTeamPoints(p1->getPoints());
	}
	server.send(200);
}

void handlePass() {
	if (server.hasArg("playerPass")) {

	}
	// TODO advance player
	//Serial.print("Player turn was = ");
	int playerid = game->getPlayerTurn();
	//Serial.println(playerid);
	Player * p1 = game->updatePlayerById(playerid);
	p1->setPassed(1);
	p1->setPlayerMsg(3);
	int turn = game->advanceNextTurn();
	//Serial.print("Player turn is now = ");
	//Serial.println(turn);

	server.send(200);
}

void handleNextHand() {
	game->moveNextState();
	server.send(200);
}

void handleGameEnd() {
	if (game->getState() == GWINNER) {
		game->setState(END);
	}
	server.send(200);
}

void handlePlayCard() {

	Serial.println("In handlePlayCard() *****************************************");

	if (server.hasArg("cardPlayed")) {
		String cardValue = server.arg("cardPlayed");
		String playerid = server.arg("pid");
		Serial.print("Player id = ");
		Serial.print(playerid);
		Serial.print(" played Card = ");
		Serial.println(cardValue);

		Player * p1 = game->updatePlayerById(playerid.toInt());
		if (p1->findCardNameByHand(game->getHandNumber()) == "back") {

			p1->updateCardPlayed(cardValue, game->getHandNumber());
			p1->setPlayerMsg(PLAYED);
			// TODO check is partner played
			String calledAce = "A" + game->getCalledSuit().substring(0, 1);
			Serial.println("****************************************");
			Serial.print("Looking for partner - player card = ");
			Serial.print(cardValue);
			Serial.print(" called Ace = ");
			Serial.println(calledAce);

			if (cardValue == calledAce) {
				game->setPartner(playerid.toInt());
				Serial.print("Setting partner = ");
				Serial.println(playerid.toInt());
			}
			Serial.println("****************************************");

			int turn = game->advanceNextTurn();
			//Serial.print("Player turn is now = ");
			//Serial.println(turn);
			server.send(200);
		}
		server.send(400);
	}
	Serial.println("In handlePlayCard() *****************************************");

}

void handleLogin() {

	//playerMessage = "Waiting for more players to join";
	//server.sendHeader("Cache-Control", "no-cache");
	//server.sendHeader("Cache-Control", "max-age=0");
	server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
	handleDownload("/login.html", "text/html");

}

//root page can be accessed only if authentification is ok
void handleRoot() {
	Serial.println("Enter handleRoot");
	//handleJson();
	String header;
	if (!is_authentified()) {
		String header = "HTTP/1.1 301 OK\r\nLocation: /login\r\nCache-Control: no-cache\r\n\r\n";
		server.sendContent(header);
		return;
	}

	String content = "<html><body><H2>Welcome ";
	Player p1 = game->getPlayerById(currentPlayer);
	content += p1.getName();
	content += " to Sheapshead 1.0! <a href=\"/login?DISCONNECT=";
	content += p1.getName();
	content += "&SESSIONID=";
	content += currentPlayer;
	content += "\">quit</a></H2><br>";

	//  if (server.hasHeader("User-Agent")) {
	//    content += "the user agent used is : " + server.header("User-Agent") + "<br><br>";
	//  }
	//  content += "You can access this page until you <a href=\"/login?DISCONNECT=";
	//  content += player[currentPlayer].playername;
	//  content += "\">disconnect</a><br>";

	for (int i = 0; i < game->getMaxPlayers(); i++) {
		content += "Player ";
		content += i;
		content += " <b>";
		Player p1 = game->getPlayerById(i);
		if (p1.getActive() == 1)
			content += p1.getName();
		else
			content += "Need Player";
		content += "</b> ";
	}
	content += "</body></html>";
	server.send(200, "text/html", content);
}

void handleLoginJson() {

	bool firstPlayer = true;
	Serial.println("In handleLoginJson");

	String message;
	message = "{";

	message += "\"pid\":";
	if (is_authentified()) {
		message += currentPlayer;
		message += ",";
		message += "\"pname\":\"";
		message += game->getPlayerById(currentPlayer).getName();
		message += "\"";
	}
	else {
		message += -1;
	}
	message += ",";

	message += "\"maxPlayers\":";
	message += game->getMaxPlayers();
	message += ",";

	Serial.print("playerMessage = (");
	Serial.print(playerMessage.substring(0, 60));
	Serial.println(")");


	message += "\"playerMessage\":\"";
	message += playerMessage.substring(0, 60);
	message += "\",";


	message += "\"numPlayers\":";
	message += game->getNumPlayers();
	message += ",";


	message += game->getPlayerJSON();


	message += "}";

	Serial.println(message);
	server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
	server.send(200, "application/json", message);
}

void handleGameJson() {

	Serial.println("In handleGameJson");

	String message;

	message = "{";

	message += "\"pid\":";
	if (is_authentified()) {
		message += currentPlayer;
	}
	else {
		message += -1;
	}
	message += ",";

	if (currentPlayer >= 0) {
		message += game->getStatJSON(currentPlayer);

		Player p1 = game->getPlayerById(currentPlayer);
		if (game->getPicker() == -1 || (!p1.getPicked() && !p1.getPassed() && game->getState() == PICKING)) {
			message += ",";
			message += game->getBlindJSON();
		}
	}

	// TODO if in the HAND* state get the cards played so far.
	if (game->getState() >= PICKING) {
		message += ",";
		message += game->getPlayerJSON();
	}

	message += "}";

	Serial.println(message);
	server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
	server.send(200, "application/json", message);
}


void handlePlayerJson() {

	Serial.print("In handlePlayerJson");
	Serial.println(server.uri());


	String message;

	message = "{";

	message += "\"pid\":";
	if (is_authentified()) {
		message += currentPlayer;
	}
	else {
		message += -1;
	}
	message += ",";

	message += game->getPlayerJSON();


	Serial.print("Game Started = ");
	Serial.println(game->isStarted());

	if (game->isStarted()) {

		Serial.println("Game Started!!!");
		message += ",";
		message += game->getStatJSON(currentPlayer);
		message += ",";
		message += game->getHandJSON(currentPlayer);

		// TODO check if player picked
		//if(game->getPicker() == currentPlayer) {
		message += ",";
		message += game->getBlindJSON();
		//}

	}
	message += "}";

	Serial.println(message);
	server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
	server.send(200, "application/json", message);
}

//no need authentification
void handleNotFound() {
	String message = "File Not Found\n\n";
	message += "URI: ";
	message += server.uri();
	message += "\nMethod: ";
	message += (server.method() == HTTP_GET) ? "GET" : "POST";
	message += "\nArguments: ";
	message += server.args();
	message += "\n";
	for (uint8_t i = 0; i < server.args(); i++) {
		message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
	}
	server.send(404, "text/plain", message);
}


int findFreeSession(String userName)
{

	//  int id = -1;
	for (int i = 0; i < game->getMaxPlayers(); i++)
	{
		//Serial.print("Session id = ");
		//Serial.print(i);

		Player p1 = game->getPlayerById(i);
		if (p1.getActive() == false)
		{

			for (int x = 0; x < game->getMaxPlayers(); x++)
			{
				Player p2 = game->getPlayerById(x);
				if (p2.getName() == userName)
				{
					//Serial.print(" with user name ");
					//Serial.print(userName);
					//Serial.println(" is used pick another");
					return -2;
				}
			}
			//Serial.println(" is not active - using");
			return i;
		}
		//Serial.print(" is active with user ");
		//Serial.println(p1.getName());
	}

	//  if ( id == -1)
	//    return -1;
	//
	//  for (int i = 0; i < MAX_PLAYERS; i++)
	//  {
	//    if (player[i].playername == userName)
	//      return -2;
	//  }

	return -1;
}

int findSessionIdbyName(String name)
{

	for (int i = 0; i < game->getMaxPlayers(); i++)
	{
		if (game->getPlayerById(i).getName() == name)
		{
			return i;
		}
	}

	return -1;
}

String findNamebySessionId(int id)
{
	if (id >= 0 && id < game->getMaxPlayers()) {
		return game->getPlayerById(id).getName();
	}
	return "";

}

/*
void initSession()
{

	for (int i = 0; i < MAX_PLAYERS; i++)
	{
		Player p1 = game->getPlayerById(i);
		//game->deletePlayer(i, "");
		p1.setActive(false);
		p1.setName("");
	}

}
*/

void handleGameStart() {

	/*
	Serial.print("In handleGame with url = ");
	Serial.print(server.uri());
	Serial.print(" and args = ");
	Serial.println(server.args());
	Serial.print("Arg START = ");
	Serial.println(server.arg("START"));
	Serial.print("Arg SESSIONID = ");
	Serial.println(server.arg("SESSIONID"));
	*/
	String header;
	if (!is_authentified()) {
		//String header = "HTTP/1.1 301 OK\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
		//server.sendContent(header);
		//server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
		//handleDownload("login.html", "text/html");
		handleLogin();
		return;
	}

	if (!game->isStarted()) {
		//game->play();
		//game->gameStarted = true;
		game->setStarted(true);
	}
	server.sendHeader("Cache-Control", "max-age=0, must-revalidate, private");
	//server.sendHeader("Cache-Control", "no-cache");
	handleDownload(server.uri() + ".html", "text/html");
}

void handleJS() {
	if (server.uri().endsWith("min.js")) {
		server.sendHeader("Cache-Control", "max-age=31536000");
	}
	handleDownload(server.uri(), "application/javascript");
}

void handleIMAGE() {
	server.sendHeader("Cache-Control", "max-age=31536000");
	handleDownload(server.uri(), "image/png");
	//server.serveStatic(server.uri().c_str(), SPIFFS, server.uri().c_str(), " max-age=86400");
}
void handleDownload(String filename, String contentType) {

	//Serial.print("In handleDownload()");

	if (!SPIFFS.begin()) {
		Serial.println("SPIFFS failed to mount !\r\n");
	}
	else {
		//Serial.println(filename);
		File file = SPIFFS.open(filename, "r");
		if (!file) {
			Serial.println("Can't open SPIFFS file !\r\n");
		}
		else {
			server.streamFile(file, contentType);
			file.close();
		}
	}
	//Serial.print("End handleDownload()");
}
